require 'backup_foundation/version'
require 'backup_foundation/item/items'
require 'backup_foundation/job'
if defined?(Rails)
  require 'backup_foundation/railtie'
end

module BackupFoundation
  HOST = 'https://backup.foundation' # 'http://localhost:3000'

  class << self
    attr_reader :app_key, :items, :encryption_key

    def store_app(app_key, &block)
      @app_key = app_key
      @items = []
      instance_eval(&block) if block_given?
    end

    def encrypt(key)
      @encryption_key = key
    end

    def backup_db
      Item::DATABASES.each do |key, val|
        if val.respond_to?(:exist?) && val.exist?(Rails.env.to_s)
          @items.push({ type: key }.merge(val.get_config(Rails.env.to_s)))
        end
      end
    end

    def backup_directory(item_name, path=nil)
      unless path
        path      = item_name
        item_name = item_name.gsub '/', '_'
      end

      @items.push type: 'directory', name: item_name, path: path
    end

    def make_backup!
      Job.new(
        key:            @app_key,
        dbs:            @items,
        encryption_key: @encryption_key
      ).backup!
    end

    def restore_backup!
      Job.new(
        key:            @app_key,
        dbs:            @items,
        encryption_key: @encryption_key
      ).restore!
    end
  end
end
