module BackupFoundation
  module Item
    class Base
      def initialize(options, tmpdir, encryption_key=nil)
        @options        = options
        @encryption_key = encryption_key
        @tmpdir         = tmpdir
      end

      def outfile_path
        "#{@tmpdir}/#{File.basename(@options[:database] || @options[:name])}.gz#{'.gpg' if @encryption_key}"
      end

      def output_to_file(command)
        "#{command} > #{outfile_path}"
      end

      def dump_and_encrypt_if_needed(command)
        if @encryption_key
          IO.pipe do |rp, wp|
            wp.puts @encryption_key
            system output_to_file("#{command} | gzip -cf | gpg --yes --batch --passphrase-fd=3 -q -r -e -c"), 3 => rp
          end
        else
          `#{output_to_file("#{command} | gzip -cf")}`
        end
      end

      def decrypt_if_needed_and_restore(command, infile_path)
        if @encryption_key
          IO.pipe do |rp, wp|
            wp.puts @encryption_key
            system "cat #{infile_path} | gpg --yes --batch --passphrase-fd=3 -q -r -d | gunzip -c | #{command}", 3 => rp
          end
        else
          `gunzip -c #{infile_path} | #{command}`
        end
      end
    end
  end
end
