module BackupFoundation
  module Item
    class MySQL < Base
      class << self
        def exist?(rails_env=nil)
          defined?(ActiveRecord::Base) &&
          ActiveRecord::Base.configurations &&
          ActiveRecord::Base.configurations[rails_env] &&
          %w(mysql mysql2 mysql2spatial).include?(ActiveRecord::Base.configurations[rails_env]['adapter'])
        end

        def get_config(rails_env=nil)
          config = ActiveRecord::Base.configurations[rails_env]
          {
            host: config['host'],
            port: config['port'],
            user: config['username'],
            password: config['password'],
            database: config['database'],
            socket: config['socket']
          }
        end
      end

      def save_dump(tmpdir)
        dump_and_encrypt_if_needed "mysqldump #{command_params} --single-transaction #{@options[:database]}"
      end

      def command_params
        [:host, :port, :socket, :user, :password].map do |option|
          next if @options[option].blank?
          "--#{option}='#{@options[option]}'"
        end.compact.join ' '
      end

      def load_dump(infile_path)
        decrypt_if_needed_and_restore "mysql #{command_params} #{@options[:database]}", infile_path
      end
    end
  end
end
