module BackupFoundation
  module Item
    class PostgreSQL < Base
      class << self
        def exist?(rails_env=nil)
          defined?(ActiveRecord::Base) &&
          ActiveRecord::Base.configurations &&
          ActiveRecord::Base.configurations[rails_env] &&
          ActiveRecord::Base.configurations[rails_env]['adapter'] == 'postgresql'
        end

        def get_config(rails_env=nil)
          config = ActiveRecord::Base.configurations[rails_env]
          {
            host: config['host'] || config['socket'],
            port: config['port'],
            username: config['username'],
            password: config['password'],
            database: config['database']
          }
        end
      end

      def save_dump(tmpdir)
        dump_and_encrypt_if_needed "#{command_password_variable} pg_dump #{command_params} #{@options[:database]}"
      end

      def command_params
        [:host, :port, :username].map do |option|
          next if @options[option].blank?
          "--#{option}='#{@options[option]}'"
        end.compact.join ' '
      end

      def command_password_variable
        @options[:password].blank? ? '' : "PGPASSWORD='#{@options[:password]}'"
      end

      def load_dump(infile_path)
        decrypt_if_needed_and_restore "#{command_password_variable} psql #{command_params} #{@options[:database]}", infile_path
      end
    end
  end
end
